package errorlink;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.ListEditor;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.externaltools.internal.model.ExternalToolsPlugin;

public class PreferencePage
	extends FieldEditorPreferencePage
	implements IWorkbenchPreferencePage
{

	private static final String REGEXP_PREF = "errorlink.regexp";
	private static String regexpListDelim = "+*";

	private static String listToString(String[] items)
	{
		StringBuffer buf = new StringBuffer();
		for (int i = 0; i < items.length; i++)
		{
			buf.append(items[i]);
			buf.append(regexpListDelim);
		}
		return buf.toString();
	}

	private static String[] stringToList(String stringList)
	{
		List result = new ArrayList();
		int idx = stringList.indexOf(regexpListDelim);
		int lastIdx = 0;
		while (idx != -1)
		{
			String item = stringList.substring(lastIdx, idx);
			result.add(item);
			lastIdx = idx + regexpListDelim.length();
			idx = stringList.indexOf(regexpListDelim, lastIdx);
		}
		return (String[]) result.toArray(new String[result.size()]);
	}

	private static IInputValidator regexpValidator = new IInputValidator()
	{
		public String isValid(String newText)
		{
			try
			{
				Pattern.compile(newText);
			}
			catch (PatternSyntaxException e)
			{
				return "Invalid regular Expression";
			}
			return null;
		}
	};

	private static class RegexpListEditor extends ListEditor
	{
		RegexpListEditor(String name, String labelText, Composite parent)
		{
			super(name, labelText, parent);
		}

		/**
		 * @see org.eclipse.jface.preference.ListEditor#createList(java.lang.String)
		 */
		protected String createList(String[] items)
		{
			return listToString(items);
		}

		/**
		 * @see org.eclipse.jface.preference.ListEditor#getNewInputObject()
		 */
		protected String getNewInputObject()
		{
			String regexp = null;
			InputDialog dialog =
				new InputDialog(
					getShell(),
					"Enter a regular expression",
					"Enter a regular expression to use for matching console output lines.",
					"",
					regexpValidator);

			int dlgResult = dialog.open();
			if (dlgResult == InputDialog.OK)
			{
				regexp = dialog.getValue();
				if (regexp.length() == 0)
					regexp = null;
			}
			return regexp;
		}

		/**
		 * @see org.eclipse.jface.preference.ListEditor#parseString(java.lang.String)
		 */
		protected String[] parseString(String stringList)
		{
			return stringToList(stringList);
		}
	}

	public PreferencePage()
	{
		super(FieldEditorPreferencePage.GRID);
		setDescription(
			"Configure the regular expressions used to create hyperlinks for Console output lines.\n\n"
				+ "Each regexp can have up to 3 groups, where the first group will expand to "
				+ "the file name, the second to the line number, and the third to the column number (currently unused) "
				+ "of the error.  Only the first group is needed to create the hyperlink.  The "
				+ "hyperlink will span the text from the start of the first regexp to the end of "
				+ "the third.  If you need to use extra groupings for your regexp, you may use the non-capturing "
				+ "form to indicate that it isn't counted - e.g. (?:some_regexp) instead of (some_regexp)\n\n"
				+ "Each console output line will be matched against each regexp in the order defined in this list.\n\n"
				+ "The default value matches jikes error messages when the ant property 'build.compiler.emacs' is on");

		// Set the preference store for the preference page.
		IPreferenceStore store =
			ExternalToolsPlugin.getDefault().getPreferenceStore();
		setPreferenceStore(store);
	}

	static void setPreferenceStoreDefaults()
	{
		IPreferenceStore store =
			ExternalToolsPlugin.getDefault().getPreferenceStore();
		store.setDefault(
			REGEXP_PREF,
			RegexpLineTracker.JIKES_ERROR_REGEXP + regexpListDelim);
	}

	public void init(IWorkbench workbench)
	{
	}

	protected void createFieldEditors()
	{
		addField(
			new RegexpListEditor(
				REGEXP_PREF,
				"Regexp",
				getFieldEditorParent()));
	}

	static public String[] getRegexps()
	{
		return stringToList(
			ExternalToolsPlugin.getDefault().getPreferenceStore().getString(
				REGEXP_PREF));
	}
}
